@@ -22,13 +22,15 @@ def launch_index(arguments):
                           genome_database=arguments.DB,
                           cut_format=arguments.rrbs_cut_format,
                           lower_bound=arguments.rrbs_lower,
-                          upper_bound=arguments.rrbs_upper)
+                          upper_bound=arguments.rrbs_upper,
+                          block_size=arguments.B)
         index.generate_rrbs_database()
     else:
         print(f'Generating WGBS Database at {arguments.DB}')
         index = WholeGenomeBuild(reference_file=arguments.G,
                                  genome_database=arguments.DB,
-                                 mappable_regions=arguments.MR)
+                                 mappable_regions=arguments.MR,
+                                 block_size=arguments.B)
         index.generate_bsb_database()
 
 
@@ -66,7 +68,7 @@ def launch_alignment(arguments):
     bsb_command_dict = {arg[0]: str(arg[1]) for arg in arguments._get_kwargs()}
     bwa_cmd = [bwa_path, 'mem', '-Y']
     if bsb_command_dict['UN'] == 'True':
-        bwa_cmd.extend(['-z', '1'])
+        bwa_cmd.extend(['-z'])
     bool_args = ['M', 'S', 'j', 'p']
     for arg in bool_args:
         if bsb_command_dict[arg] == 'True':
@@ -82,6 +84,9 @@ def launch_alignment(arguments):
         bwa_cmd.extend([f'-O', bsb_command_dict['INDEL']])
     if bsb_command_dict['XA']:
         bwa_cmd.extend([f'-h', bsb_command_dict['XA']])
+    bwa_cmd.extend(['-l', bsb_command_dict['CP']])
+    bwa_cmd.extend(['-n', bsb_command_dict['CT']])
+    bwa_cmd.extend(['-Z', bsb_command_dict['DR']])
     database = bsb_command_dict['DB']
     if not database.endswith('.fa'):
         if not database.endswith('/'):
@@ -89,12 +94,13 @@ def launch_alignment(arguments):
         else:
             database = f'{database}BSB_ref.fa'
         assert os.path.exists(database), f'-DB {arguments.DB} does not exist, please index genome'
+        assert os.path.exists(f'{database}.opac'), f'-DB {arguments.DB} not complete, please re-index genome'
     bwa_cmd.append(database)
     bwa_cmd.append(bsb_command_dict['F1'])
     assert os.path.exists(arguments.F1), f'-F1 {arguments.F1} does not exits, please check path'
     if bsb_command_dict['F2'] != 'None':
         bwa_cmd.append(bsb_command_dict['F2'])
-        assert os.path.exists(arguments.F1), f'-F2 {arguments.F2} does not exits, please check path'
+        assert os.path.exists(arguments.F2), f'-F2 {arguments.F2} does not exits, please check path'
     align_bisulfite(bwa_cmd, arguments.O)
 
 
