@@ -1,6 +1,7 @@
 import argparse
 
 from BSBolt.Utils.UtilityFunctions import get_external_paths
+from BSBolt.Utils.ParserHelpMessages import aggregate_help, alignment_help, impute_help, index_help, meth_help, sim_help
 
 bt2_path, art_path = get_external_paths()
 
@@ -12,26 +13,32 @@ subparsers = parser.add_subparsers(description='BSBolt Modules, Please Invoke BS
                                    metavar='Index, Align, CallMethylation, AggregateMatrix, Simulate, Impute',
                                    dest='subparser_name')
 
-align_parser = subparsers.add_parser('Align', help='Alignment Module')
-index_parser = subparsers.add_parser('Index', help='Index Generation Module')
-call_meth_parser = subparsers.add_parser('CallMethylation', help='Methylation Calling Module')
-matrix_parser = subparsers.add_parser('AggregateMatrix', help='CGmap Matrix Aggregation Module')
-sim_parser = subparsers.add_parser('Simulate', help='BSBolt Illumina Read Simulation Module')
-imputation_parser = subparsers.add_parser('Impute', help='kNN Imputation Module')
+align_parser = subparsers.add_parser('Align', help='Alignment Module', usage=alignment_help, add_help=False)
+index_parser = subparsers.add_parser('Index', help='Index Generation Module', usage=index_help, add_help=False)
+call_meth_parser = subparsers.add_parser('CallMethylation', help='Methylation Calling Module',
+                                         usage=meth_help, add_help=False)
+matrix_parser = subparsers.add_parser('AggregateMatrix', help='CGmap Matrix Aggregation Module', add_help=False,
+                                      usage=aggregate_help)
+sim_parser = subparsers.add_parser('Simulate', help='Read Simulation Module', add_help=False, usage=sim_help)
+imputation_parser = subparsers.add_parser('Impute', help='kNN Imputation Module', add_help=False, usage=impute_help)
 sort_parser = subparsers.add_parser('Sort', help='BAM Sort')
 bam_index = subparsers.add_parser('BamIndex', help='BAM Index')
 # Add Alignment Parser Commands
 
-align_parser.add_argument('-F1', type=str, default=None, help='Path to fastq 1', required=True)
-align_parser.add_argument('-F2', type=str, default=None, help='Path to fastq 2')
+align_parser.add_argument('-F1', type=str, default=None, help='path to fastq 1', required=True)
+align_parser.add_argument('-F2', type=str, default=None, help='path to fastq 2')
 align_parser.add_argument('-UN', action="store_true", default=False,
-                          help='Library Undirectional, Consider PCR products of bisulfite converted DNA',
+                          help='library undirectional, ie. consider PCR products of bisulfite converted DNA',
                           required=False)
-align_parser.add_argument('-O', type=str, default=None, help='Path to Output Prefix', required=True)
-align_parser.add_argument('-DB', type=str, default=None, help='Path to BSBolt Database', required=True)
-align_parser.add_argument('-t', type=int, default=1, help='Number of bwa threads [1]', required=False)
-align_parser.add_argument('-k', type=int, default=19, help='Minimum seed length [19]', required=False)
-align_parser.add_argument('-w', type=int, default=100, help='Band width for banded alignment [100]', required=False)
+align_parser.add_argument('-O', type=str, default=None, help='output Prefix', required=True)
+align_parser.add_argument('-DB', type=str, default=None, help='path to BSBolt database', required=True)
+align_parser.add_argument('-CP', type=float, default=0.5, help='CH conversion proportion threshold [0.5]',
+                          required=False)
+align_parser.add_argument('-CT', type=int, default=5, help='number of CH sites needed to assess read conversion',
+                          required=False)
+align_parser.add_argument('-t', type=int, default=1, help='number of bwa threads [1]', required=False)
+align_parser.add_argument('-k', type=int, default=19, help='minimum seed length [19]', required=False)
+align_parser.add_argument('-w', type=int, default=100, help='band width for banded alignment [100]', required=False)
 align_parser.add_argument('-d', type=int, default=100, help='off-diagonal X-dropoff [100]', required=False)
 align_parser.add_argument('-r', type=float, default=1.5,
                           help='look for internal seeds inside a seed longer than {-k} * FLOAT [1.5]', required=False)
@@ -87,6 +94,7 @@ align_parser.add_argument('-XA', type=lambda x: x.strip(), default='100,200',
                           help='if there are <INT hits with score >80 percent of the max score, '
                                'output all in XA [100,200]',
                           required=False)
+align_parser.add_argument('-DR', type=float, default=0.95, help='drop ratio for alternative hits reported in XA tag, for best bisulifte alignment performance set at or above default [0.95]')
 align_parser.add_argument('-M', action='store_true', default=False,
                           help='mark shorter split hits as secondary',
                           required=False)
@@ -103,6 +111,9 @@ index_parser.add_argument('-G', type=str, required=True,
                           help='Path to reference genome fasta file, fasta file should contain all contigs')
 index_parser.add_argument('-DB', type=str, required=True,
                           help='Path to index directory, will create directory if folder does not exist')
+index_parser.add_argument('-B', type=int, default=10000000, help='Block size for bwtsw algorithm, increasing '
+                                                                 'block will speed up indexing and increase memory '
+                                                                 'consumption')
 index_parser.add_argument('-MR', type=str, default=None, help='Path to bed file of mappable regions. Index will be '
                                                               'built using using masked contig sequence')
 index_parser.add_argument('-rrbs', action="store_true", default=False, help='Generate a Reduced Representative'
@@ -123,15 +134,13 @@ call_meth_parser.add_argument('-I', type=str, required=True,
 call_meth_parser.add_argument('-DB', type=str, required=True, help='Path to index directory')
 call_meth_parser.add_argument('-O', type=str, required=True, help='Output prefix')
 call_meth_parser.add_argument('-remove-ccgg', action="store_true", default=False,
-                              help='Remove methylation calls in ccgg sites,'
-                                   'default=False')
-call_meth_parser.add_argument('-verbose', action="store_true", default=False, help='Verbose Output, default=False')
+                              help='Remove methylation calls in ccgg sites')
+call_meth_parser.add_argument('-verbose', action="store_true", default=False, help='Verbose Output')
 call_meth_parser.add_argument('-text', action="store_true", default=False,
                               help='Output plain text files, default=False')
-call_meth_parser.add_argument('-ignore-overlap', action="store_true", default=False,
+call_meth_parser.add_argument('-ignore-ov', action="store_true", default=True,
                               help='Only consider higher quality base '
-                                   'when paired end reads overlap, '
-                                   'default=False')
+                                   'when paired end reads overlap')
 call_meth_parser.add_argument('-max', type=int, default=8000, help='Max read depth to call methylation')
 call_meth_parser.add_argument('-min', type=int, default=10,
                               help='Minimum read depth required to report methylation site')
@@ -179,28 +188,28 @@ sim_parser.add_argument('-G', type=str, required=True,
                         help='Path for reference genome fasta file, fasta file should contain all contigs')
 sim_parser.add_argument('-O', type=str, required=True, help='Output prefix')
 sim_parser.add_argument('-PE', default=False, action='store_true', help='Simulate Paired End Reads, default Single End')
-sim_parser.add_argument('-RL', type=int, default=125, help='Simulated Read Length')
-sim_parser.add_argument('-RD', type=int, default=20, help='Simulated Read Depth')
+sim_parser.add_argument('-RL', type=int, default=125, help='Simulated Read Length [125]')
+sim_parser.add_argument('-RD', type=int, default=20, help='Simulated Read Depth [20]')
 sim_parser.add_argument('-U', default=False, action='store_true',
                         help='Simulate Undirectional Reads, default=Directional')
 sim_parser.add_argument('-CG', default=None, help='Path to CGmap file to generate simulation reference profile')
 sim_parser.add_argument('-BR', default=None, help='Path to previously generated BSBolt methylation reference')
-sim_parser.add_argument('-MR', type=float, default=0.005, help='Mutation rate')
-sim_parser.add_argument('-MI', type=float, default=0.20, help='Mutation indel fraction')
-sim_parser.add_argument('-ME', type=float, default=0.20, help='Mutation indel extension probability')
-sim_parser.add_argument('-RS', type=int, default=-1, help='Random seed for variant generation')
+sim_parser.add_argument('-MR', type=float, default=0.005, help='Mutation rate [0.005]')
+sim_parser.add_argument('-MI', type=float, default=0.20, help='Mutation indel fraction [0.20]')
+sim_parser.add_argument('-ME', type=float, default=0.20, help='Mutation indel extension probability [0.20]')
+sim_parser.add_argument('-RS', type=int, default=-1, help='Random seed for variant generation [-1]')
 sim_parser.add_argument('-HA', default=False, action='store_true', help='Haplotype mode, homozygous variants only')
 sim_parser.add_argument('-CH', default=True, action='store_false', help='Skip simulation of CH methylation, '
                                                                         'all CH sites unmethylated')
 sim_parser.add_argument('-NS', default=True, action='store_false', help='By default observed methylation counts are '
                                                                         'saved, disable this behavior')
-sim_parser.add_argument('-SE', type=float, default=0.001, help='Sequencing Error')
+sim_parser.add_argument('-SE', type=float, default=0.001, help='Sequencing Error [0.001]')
 sim_parser.add_argument('-NF', type=float, default=0.05, help='Cutoff threshold for amibiguous bases, simulated reads '
                                                               'with a proportion of ambiguous bases above this '
-                                                              'threshold will not be output')
-sim_parser.add_argument('-FM', type=int, default=400, help='Max fragment size')
-sim_parser.add_argument('-IM', type=int, default=50, help='Insert length mean')
-sim_parser.add_argument('-SM', type=int, default=50, help='Insert length standard deviation')
+                                                              'threshold will not be output [0.05]')
+sim_parser.add_argument('-FM', type=int, default=400, help='Max fragment size [400]')
+sim_parser.add_argument('-IM', type=int, default=50, help='Insert length mean [50]')
+sim_parser.add_argument('-SM', type=int, default=50, help='Insert length standard deviation [50]')
 sim_parser.add_argument('-verbose', action='store_true', default=False, help='Verbose read simulation')
 sim_parser.add_argument('-overwrite', action='store_true', default=False, help='Overwrite previously generated '
                                                                                'simulation database')
